package com.starbuds.server.service.demo._rocketmqons;

import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.Producer;
import com.aliyun.openservices.ons.api.SendResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @author gy
 * @className OnsProducer
 * @description
 * @date 2023/6/24
 * 封装一个发送消息的方法。
 * <p>
 * rocketons中StringBuilder logsb = new StringBuilder("send OnsMQ Msg:");
 * Message msg = getInstance(topic, tag, msgStr);
 * Producer producer = CloudMQUtil.getProducer();
 * SendResult sendResult = producer.send(msg); //然后报错
 * 2023-06-24 20:49:23 CST tm-demo ERROR org.apache.juli.logging.DirectJDKLog 175 log - Servlet.service() for servlet [dispatcherServlet] in context with path [] threw exception [Request processing failed; nested exception is com.aliyun.openservices.ons.api.exception.ONSClientException: Receive a broker exception, Topi=TPC_NORMAL, msgId=null, valid resource owner failed. maybe the resource MQ_INST_1874224963112724_BX9XLUKJ%TPC_NORMAL not created
 * See http://docs.aliyun.com/cn#/pub/ons/faq/exceptions&broker_response_exception for further details.] with root cause com.aliyun.openservices.ons.api.exception.ONSClientException: Receive a broker exception, Topi=TPC_NORMAL, msgId=null, valid resource owner failed. maybe the resource MQ_INST_1874224963112724_BX9XLUKJ%TPC_NORMAL not created
 * <p>
 * properties.put(PropertyKeyConst.GROUP_ID, MqConfigParams.GROUP_ID);
 * properties.put(PropertyKeyConst.AccessKey, MqConfigParams.ACCESS_KEY);
 * properties.put(PropertyKeyConst.SecretKey, MqConfigParams.SECRET_KEY);
 * properties.put(PropertyKeyConst.NAMESRV_ADDR, MqConfigParams.NAMESRV_ADDR);
 * 对应属性都有值，topic也
 */
@Service
@Slf4j
public class OnsProducer {

    public void sendMessage(String topic, String tag, String msgStr) {
        StringBuilder logsb = new StringBuilder("send OnsMQ Msg:");
        Message msg = getInstance(topic, tag, msgStr);
        Producer producer = CloudMQUtil.getProducer();
        SendResult sendResult = producer.send(msg);
        String messageId = "";
        messageId = sendResult.getMessageId();
        if (messageId != null && !messageId.equals("")) {
            logsb.append("生产者[OnsProducer] : " + "{ messageId : " + messageId + " , msgStr :" + msgStr + "}");
        } else {
            logsb.append("生产者[OnsProducer] fail: " + "{ " + " msgStr :" + msgStr + "}");
        }
        log.info(logsb.toString());
    }

    private static Message getInstance(String topic, String tag, String body) {
        if (body.equals("") || body == null) {
            body = "";
        }
        Message msg = new Message(topic, tag, body.getBytes());
        return msg;
    }

}
