package com.starbuds.server.service.demo.data.dao;

import com.starbuds.server.common.demo.BookDO;
import com.starbuds.server.common.demo.BookVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 书籍持久层
 *
 * @author gy
 * @date 2023/9/20
 */
@Mapper
@Repository
public interface BookDAO {
    /**
     * 插入
     *
     * @param bookDO
     * @return
     */
    int insert(BookDO bookDO);

    /**
     * 删除书籍通过书籍id
     *
     * @param bookId
     * @return
     */
    int deleteById(@Param("bookId") Long bookId);

    /**
     * 更新书籍
     *
     * @param bookId
     * @param bookName
     * @param bookAuthor
     * @param updateTime
     * @return
     */
    int update(@Param("bookId") long bookId,
               @Param("bookName") String bookName,
               @Param("bookAuthor") String bookAuthor,
               @Param("updateTime") long updateTime);

    /**
     * 获取书籍列表
     *
     * @param bookType 为空时获取全部书籍
     * @param bookName 为空时获取全部书籍
     * @param pageCursor
     * @param pagesize
     * @return
     */
    List<BookVO> selectList(@Param("bookType") Integer bookType,
                            @Param("bookName") String bookName,
                            @Param("pageCursor") Integer pageCursor,
                            @Param("pageSize") long pagesize);

    /**
     * 获取作者所有的书对应的音乐列表
     *
     * @param bookAuthor
     * @param pageCursor
     * @param pagesize
     * @return
     */
    List<BookVO> selectListJoin(@Param("bookAuthor") String bookAuthor,
                                @Param("pageCursor") Integer pageCursor,
                                @Param("pageSize") long pagesize);

    /**
     * 获取书籍通过id
     *
     * @param bookId
     * @return
     */
    BookDO selectById(@Param("bookId") long bookId);

    /**
     * 获取书籍数量
     *
     * @param bookType
     * @return
     */
    Integer getTotalCount(@Param("bookType") Integer bookType);
}
