package com.starbuds.server.service.demo.impl;

import com.starbuds.server.common.demo.BookDO;
import com.starbuds.server.common.demo.BookDemoService;
import com.starbuds.server.common.demo.BookSearchDemoService;
import com.starbuds.server.common.demo.BookVO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.service.demo.data.dao.BookDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * 书籍服务
 *
 * @author gy
 * @date 2023/9/18
 */
@Slf4j
@Service(version = "1.0.0")
public class BookDemoServiceImpl implements BookDemoService {

    @Resource
    BookDAO bookDAO;

    @Resource
    BookSearchDemoService bookSearchDemoService;

    /**
     * 增加书籍
     *
     * @param bookDO
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addBook(BookDO bookDO) {
        int count = bookDAO.insert(bookDO);
        if(count > 0){
            bookSearchDemoService.saveOrUpdate(bookDO);
            return true;
        }
        return false;
    }

    /**
     * 删除书籍
     *
     * @param bookId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteBook(long bookId) {
        int count = bookDAO.deleteById(bookId);
        if(count > 0){
            bookSearchDemoService.deleteBook(bookId);
            return true;
        }
        return false;
    }

    /**
     * 更新书籍
     *
     * @param bookId
     * @param bookName
     * @param bookAuthor
     * @return
     */
    @Override
    public boolean updateBook(long bookId, String bookName, String bookAuthor) {
        long curentTime = System.currentTimeMillis();
        BookDO orido = bookDAO.selectById(bookId);
        BookDO bookDO = new BookDO();
        BeanUtils.copyProperties(orido,bookDO);
        if(bookId > 0){
            bookDO.setBookId(bookId);
        }
        if(bookName != null){
            bookDO.setBookName(bookName);
        }
        if(bookAuthor != null){
            bookDO.setBookAuthor(bookAuthor);
        }
        if(curentTime > 0){
            bookDO.setUpdateTime(curentTime);
        }
        // int count = bookDAO.update(bookId, bookName, bookAuthor,curentTime);
        int count = bookDAO.update(bookDO.getBookId(), bookDO.getBookName(), bookDO.getBookAuthor(),bookDO.getUpdateTime());
        if(count > 0){
            // BookDO orido = bookDAO.selectById(bookId);
            // BookDO bookDO = new BookDO();
            // BeanUtils.copyProperties(orido,bookDO);
            // if(bookName != null){
            //     bookDO.setBookName(bookName);
            // }
            // if(bookAuthor != null){
            //     bookDO.setBookAuthor(bookAuthor);
            // }
            // if(curentTime > 0){
            //     bookDO.setUpdateTime(curentTime);
            // }
            bookSearchDemoService.saveOrUpdate(bookDO);
            return true;
        }
        return false;
    }

    /**
     * 获取书籍列表
     *
     * @param bookType 为空时获取全部书籍
     * @param bookName 为空时获取全部书籍
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<BookVO> getBookList(int bookType, String bookName, int pageIndex, int pageSize) {
        PageList<BookVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(bookDAO.selectList(bookType, bookName, pageList.getPageCursor(),pageSize));
        pageList.setTotal(bookDAO.getTotalCount(bookType));
        return pageList;
    }

    /**
     * 获取作者所有的书对应的音乐列表
     *
     * @param bookAuthor
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<BookVO> getBookMusicList(String bookAuthor, int pageIndex, int pageSize) {
        PageList<BookVO> pageList = new PageList<>(pageIndex, pageSize);
        List<BookVO> bookList = bookDAO.selectListJoin(bookAuthor, pageList.getPageCursor(),pageSize);
        log.info("getPageCursor==="+pageList.getPageCursor());
        pageList.setList(bookList);
        pageList.setTotal(bookList.size());
        return pageList;
    }
}





