package com.starbuds.server.service.demo.impl;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.demo.BookDO;
import com.starbuds.server.common.demo.BookSearchDemoService;
import com.starbuds.server.common.demo.BookVO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.service.demo.es.dao.BookRepository;
import com.starbuds.server.service.demo.es.entity.BookEntity;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 书籍搜索服务
 *
 * @author gy
 * @date 2023/9/20
 */
@Slf4j
@Service(version = "1.0.0")
public class BookSearchDemoServiceImpl implements BookSearchDemoService {
    @Resource
    BookRepository bookRepository;

    /**
     * 增加/更新书籍
     *
     * @param bookDO
     * @return
     */
    @Override
    public boolean saveOrUpdate(BookDO bookDO) {
        if(bookDO == null){
            return false;
        }

        BookEntity exist = bookRepository.findById(bookDO.getBookId()).orElse(null);
        if(exist != null){
            log.info("已存在==="+exist);
        }

        BookEntity entity = new BookEntity();
        if(bookDO.getBookId() != null){
            entity.setBookId(bookDO.getBookId());
        }
        if(bookDO.getBookType() > 0){
            entity.setBookType(bookDO.getBookType());
        }
        if(StringUtils.isNotEmpty(bookDO.getBookName())){
            entity.setBookName(bookDO.getBookName());
            entity.setBookNameIK(bookDO.getBookName());
        }
        if(StringUtils.isNotEmpty(bookDO.getBookAuthor())){
            entity.setBookAuthor(bookDO.getBookAuthor());
        }
        if(bookDO.getCreateTime() > 0){
            entity.setCreateTime(bookDO.getCreateTime());
        }
        if(bookDO.getUpdateTime() > 0){
            entity.setUpdateTime(bookDO.getUpdateTime());
        }

        bookRepository.save(entity);

        return true;
    }

    /**
     * 删除书籍
     *
     * @param bookId
     * @return
     */
    @Override
    public boolean deleteBook(long bookId) {
        BookEntity exist = bookRepository.findById(bookId).orElse(null);
        if(exist == null){
            log.info("不存在==="+exist);
            throw new RuntimeException("书籍不存在");
        }

        BookEntity bookEntity = new BookEntity();
        bookEntity.setBookId(bookId);
        bookRepository.delete(bookEntity);
        // bookRepository.deleteById(bookId);//通过id直接删除
        return true;
    }

    /**
     * 搜索书籍
     *
     * @param content
     * @return
     */
    @Override
    public PageList<BookVO> searchBook(String content, int pageIndex, int pageSize) {
        PageList<BookVO> page = new PageList<>(pageIndex, pageSize);
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        if(StringUtils.isNotEmpty(content)){
            BoolQueryBuilder shouldQueryBuilder = QueryBuilders.boolQuery();
            shouldQueryBuilder.should(QueryBuilders.matchQuery("bookName", content));
            shouldQueryBuilder.should(QueryBuilders.wildcardQuery("bookName", String.format("*%s*", content)));
            shouldQueryBuilder.should(QueryBuilders.matchQuery("bookAuthor", content));
            shouldQueryBuilder.should(QueryBuilders.wildcardQuery("bookAuthor", String.format("*%s*", content)));
            boolQueryBuilder.must(shouldQueryBuilder);
        }
        // boolQueryBuilder.must(QueryBuilders.matchQuery("shelfStatus", 1));
        // boolQueryBuilder.must(QueryBuilders.rangeQuery("status").gt(MusicStatusEnum.USER_DEL.getValue()));
        log.info("查询语句：{}", boolQueryBuilder.toString());

        PageRequest pageRequest = PageRequest.of(pageIndex - 1, pageSize, Sort.by(Sort.Direction.DESC, "bookId"));
        Page<BookEntity> searchResult = bookRepository.search(boolQueryBuilder, pageRequest);
        List<BookEntity> resultContent = searchResult.getContent();
        log.info("es返回的结果内容：{}", JSON.toJSONString(resultContent, true));

        page.setTotal(Convert.toInt(searchResult.getTotalElements()));

        List<BookVO> respList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(resultContent)) {
            //List<Long> musicIds = resultContent.stream().map(MusicEntity::getMusicId).collect(Collectors.toList());
            //respList = musicDAO.selectByIds(musicIds);
            for (BookEntity entity : resultContent){
                BookVO bookVO = new BookVO();
                bookVO.setBookId(entity.getBookId());
                bookVO.setBookType(entity.getBookType());
                bookVO.setBookName(entity.getBookName());
                bookVO.setBookAuthor(entity.getBookAuthor());
                bookVO.setCreateTime(entity.getCreateTime());
                bookVO.setUpdateTime(entity.getUpdateTime());
                respList.add(bookVO);
            }
        }
        page.setList(respList);
        return page;
    }

}
