package com.starbuds.server.service.demo;

import cn.hutool.core.lang.UUID;
import com.starbuds.server.common.constant.cache.RedisChatKeys;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.demo.BookDO;
import com.starbuds.server.common.demo.BookSearchDemoService;
import com.starbuds.server.common.service.data.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Slf4j
@SpringBootTest
class DemoApplicationTests {

    // @Resource
    // BookSearchDemoService bookSearchDemoService;
    // @Test
    // void esTest(){
    //     BookDO bookDO = new BookDO();
    //     bookDO.setBookId(123123L);
    //     bookDO.setBookName("中国共产党");
    //     bookDO.setBookAuthor("毛泽东");
    //     bookSearchDemoService.saveOrUpdate(bookDO);
    // }

    @Reference(version = "1.0.0")
    RedisService redisService;
    @Test
    void contextLoads() {
        //---key string---
        redisService.delKey("stringtype1");
        log.info("hasKey="+redisService.hasKey("stringtype1"));
        redisService.expire("stringtype1", 1000000);

        redisService.set("stringtype1", "stringcontent1");
        redisService.set("stringtype2", "stringcontent2");
        log.info("string="+redisService.get("stringtype1"));

        List<String> list = new ArrayList<>();
        list.add("stringtype1");
        list.add("stringtype2");
        log.info("batchGet="+redisService.batchGet(list));

        redisService.set("stringtype1", "stringcontent3",1000000, TimeUnit.MILLISECONDS);

        // 处理 combo
        String comboKey = RedisRtcKeys.getGiftComboKey(12345678L, 123123L, 123123L, 123, "123123");
        long combo = redisService.increaseAndExpire(comboKey, 1, 100000000, TimeUnit.SECONDS);
        log.info("combo="+combo);

        //---hash---
        log.info("hashGet="+redisService.hashGet("ADMIN_TOKEN","8"));
        log.info("hashGetAll="+redisService.hashGetAll("ACTIVITY_LOTTERY_POOL"));
        redisService.hashPut("hashtype1", "hashkey1", "hashcontent1");
        redisService.hashPut("hashtype1", "hashkey2", "hashcontent20");
        log.info("hashSize="+redisService.hashSize("hashtype1"));
        log.info("hashKeys="+redisService.hashKeys("hashtype1"));

        redisService.hashPutIfAbsent("hashtype2","hashkey3","hashcontent4");//hashtype2和hashkey3不存在时写入
        redisService.hashDel("hashtype1", "hashkey2");
        //redisService.hashIncreaseLong("hashtype1","hashkey1",10);

        StringBuilder sb = new StringBuilder();
        sb.append("local t = redis.call('TIME');");
        sb.append("return t[1]");
        String luaTime = sb.toString();
        log.info("luaTime="+redisService.luaLong(luaTime, new ArrayList<>(), new ArrayList<>()));

        //下订单、竞拍，线上锁，在解锁。没有值就存，有值就标识存过，然后报错。发生错误就释放锁。
        String key = RedisChatKeys.getSkillOrderStartKey(123123l);
        String value = UUID.randomUUID().toString();
        redisService.tryLock(key, value, 10, TimeUnit.SECONDS);
        redisService.releaseLock(key, value);

        //---set---
        //set
        redisService.sAdd("settype2","setkey3");
        redisService.sAdd("settype2","setkey4");
        // redisService.sRemove("key1", "content1");
        log.info("setMembers="+redisService.setMembers("settype2"));
        log.info("sMembers="+redisService.sMembers("settype2"));

        //zset
        redisService.zAdd("settype1", "setkey1", 123d);
        redisService.zAdd("settype1", "setkey2", 12d);
        // redisService.delKey("settype1");
        // redisService.zRemove("settype1","setkey1");
        log.info("zIncrementScore="+redisService.zIncrementScore("settype1","setkey1",456d));

        //list
        List<String> rewards = new ArrayList<>();
        rewards.add("123");
        rewards.add("123");
        rewards.add("456");
        redisService.lLeftPushAll("listkey1", rewards);
        log.info("lLeftPop="+redisService.lLeftPop("listkey1"));

    }
}
